@@ -34,14 +34,15 @@ impl Connection {
     }
 
     pub async fn open_bi(&self) -> Result<(SendStream, RecvStream)> {
-        let (send_stream, recv_stream) = self.handle_error(self.quic_conn.open_bi().await)?;
+        let (send_stream, recv_stream) = self.handle_error(self.quic_conn.open_bi().await).await?;
         Ok((SendStream::new(send_stream), RecvStream::new(recv_stream)))
     }
 
     /// Send message to peer using a uni-directional stream.
     pub async fn send_uni(&self, msg: Bytes) -> Result<()> {
-        let mut send_stream = self.handle_error(self.quic_conn.open_uni().await)?;
-        self.handle_error(send_msg(&mut send_stream, msg).await)?;
+        let mut send_stream = self.handle_error(self.quic_conn.open_uni().await).await?;
+        self.handle_error(send_msg(&mut send_stream, msg).await)
+            .await?;
 
         // We try to make sure the stream is gracefully closed and the bytes get sent,
         // but if it was already closed (perhaps by the peer) then we
@@ -49,15 +50,15 @@ impl Connection {
         match send_stream.finish().await {
             Ok(()) | Err(quinn::WriteError::Stopped(_)) => Ok(()),
             Err(err) => {
-                self.handle_error(Err(err))?;
+                self.handle_error(Err(err)).await?;
                 Ok(())
             }
         }
     }
 
-    fn handle_error<T, E>(&self, result: Result<T, E>) -> Result<T, E> {
+    async fn handle_error<T, E>(&self, result: Result<T, E>) -> Result<T, E> {
         if result.is_err() {
-            self.remover.remove()
+            self.remover.remove().await
         }
 
         result
@@ -154,7 +155,7 @@ pub(super) fn listen_for_incoming_connections(
                         ..
                     }) => {
                         let peer_address = connection.remote_address();
-                        let pool_handle = connection_pool.insert(peer_address, connection);
+                        let pool_handle = connection_pool.insert(peer_address, connection).await;
                         let _ = connection_tx.send(peer_address);
                         listen_for_incoming_messages(
                             uni_streams,
@@ -166,7 +167,10 @@ pub(super) fn listen_for_incoming_connections(
                         );
                     }
                     Err(err) => {
-                        error!("An incoming connection failed because of an error: {}", err);
+                        error!(
+                            "An incoming connection failed because of an error: {:?}",
+                            err
+                        );
                     }
                 },
                 None => {
@@ -197,7 +201,7 @@ pub(super) fn listen_for_incoming_messages(
 
         log::trace!("The connection to {:?} has been terminated.", src);
         let _ = disconnection_tx.send(src);
-        remover.remove();
+        remover.remove().await;
     });
 }
 
@@ -215,7 +219,7 @@ async fn read_on_uni_streams(
             }
             Err(err) => {
                 warn!(
-                    "Failed to read incoming message on uni-stream for peer {:?} with error: {}",
+                    "Failed to read incoming message on uni-stream for peer {:?} with error: {:?}",
                     peer_addr, err
                 );
                 break;
@@ -229,7 +233,7 @@ async fn read_on_uni_streams(
                     Err(Error::StreamRead(quinn::ReadExactError::FinishedEarly)) => break,
                     Err(err) => {
                         error!(
-                            "Failed reading from a uni-stream for peer {:?} with error: {}",
+                            "Failed reading from a uni-stream for peer {:?} with error: {:?}",
                             peer_addr, err
                         );
                         break;
@@ -255,7 +259,7 @@ async fn read_on_bi_streams(
             }
             Err(err) => {
                 warn!(
-                    "Failed to read incoming message on bi-stream for peer {:?} with error: {}",
+                    "Failed to read incoming message on bi-stream for peer {:?} with error: {:?}",
                     peer_addr, err
                 );
                 break;
@@ -268,7 +272,7 @@ async fn read_on_bi_streams(
                     Ok(WireMsg::EndpointEchoReq) => {
                         if let Err(error) = handle_endpoint_echo_req(peer_addr, &mut send).await {
                             error!(
-                                "Failed to handle Echo Request for peer {:?} with error: {}",
+                                "Failed to handle Echo Request for peer {:?} with error: {:?}",
                                 peer_addr, error
                             );
                         }
@@ -282,7 +286,7 @@ async fn read_on_bi_streams(
                         )
                         .await
                         {
-                            error!("Failed to handle Endpoint verification request for peer {:?} with error: {}", peer_addr, error);
+                            error!("Failed to handle Endpoint verification request for peer {:?} with error: {:?}", peer_addr, error);
                         }
                     }
                     Ok(msg) => {
@@ -294,7 +298,7 @@ async fn read_on_bi_streams(
                     Err(Error::StreamRead(quinn::ReadExactError::FinishedEarly)) => break,
                     Err(err) => {
                         error!(
-                            "Failed reading from a bi-stream for peer {:?} with error: {}",
+                            "Failed reading from a bi-stream for peer {:?} with error: {:?}",
                             peer_addr, err
                         );
                         break;
@@ -387,6 +391,7 @@ mod tests {
 
         let connection = peer1
             .get_connection(&peer2_addr)
+            .await
             .ok_or(Error::MissingConnection)?;
         let (mut send_stream, mut recv_stream) = connection.open_bi().await?;
         let message = WireMsg::EndpointEchoReq;
